var app = angular.module('starter.services', ['ionic', 'toaster']);



app.service('TubesService', function($q, $sce, $filter, $rootScope, $http, $timeout, toaster) {
  
  var starTubes = null;

  var failed = false;

  var setStar = function(tube){
    if(starTubes == null){
      starTubes = JSON.parse(localStorage.getItem('starTubes'));
      if(!starTubes)
        starTubes = {};
    }
    if(starTubes[tube.id] != null){
      tube.star = true;
    }
  };

  var readyFn = function(curTubes, dfd){
    
    var tubes = [];
    var i = 0;
    curTubes.forEach(function(tube){
      i++;
      setStar(tube);


      ImgCache.isCached(tube.thumb, function(path, success) {
        if (success) {
          ImgCache.getCachedFileURL(tube.thumb, function(path, nurl) {
            $rootScope.$apply(function(){
              tube.thumbCache = nurl;
            })
          })
        } else {
          ImgCache.cacheFile(tube.thumb, function() {
            $rootScope.$apply(function(){
              tube.thumbCache = tube.thumb;
            })
          })
        }
      })


      tubes.push(tube);
      if(i == curTubes.length){
        dfd.resolve(tubes);
      }
    })
    return dfd.promise;
  };

  var failFn = function(curTubes){

    var tubes = [];
    var i = 0;
    ImgCache.cacheFailed = true;
    curTubes.forEach(function(tube){
      i++;
      setStar(tube);
      tubes.push(tube);
      if(i == curTubes.length){
        dfd.resolve(tubes);
      }
    })
    return dfd.promise;
  };

  var isToast = false;

  return {

    displayWarning: function(title, body){
      if(!isToast){
        isToast = true;
        toaster.pop({
          type: 'warning',
          body: body,
          title: title,
          showCloseButton: false
        });
        $timeout(function() {
          isToast = false;
        }, 3200);
      }
    },

    getStarTubes: function(){
      if(starTubes == null){
        this.starTubes = JSON.parse(localStorage.getItem('starTubes'));
        if(!this.starTubes){
          this.starTubes = {};
        }
      }
      return this.starTubes;
    },

    service: {},

    isFailed: function(){
      console.log(failed);
      if(failed){
        return true;
      }
      return false;
    },

    setIntTube: function(tube){
      if(this.service == null){
        this.service = {};
      }
      this.service.tube = tube;
    },

    getIntTube: function(tubeId){
      if(this.service && this.service.tube && this.service.tube.id == tubeId){
        return this.service.tube;
      }else {
        return this.getExtTube();
      }
    },

    getIntTubed: function(tubeId){
      return this.service.tube;
    },

    invalidateService: function(){
      this.service.invalid = true;
    },

    isFirstUse: function(){
      if(localStorage.getItem('service') === null){
        return true;
      }
      return false;
    },

    getLatest: function(){

      var dfd = $q.defer();
      var ts = this.getTubes;
      var t = this;
      $http.get('https://s3-eu-west-1.amazonaws.com/tubesmash/latest.json').success(function(data, status, headers, config) {
        
        ts(data).then(function(tubes){
          dfd.resolve(tubes);
        });
      }).error(function(data, status, headers, config) {
        dfd.reject();
        
        t.displayWarning($filter('translate')('ops'), $filter('translate')('no_internet'));
      })
      return dfd.promise;
    },

    getService: function(){
      var displayWarning = this.displayWarning;
      var failed = false;
      var dfd = $q.defer();
      if(this.service.data === undefined || this.service.invalid){
        
        var tempService = localStorage.getItem('service');
        var lastDay = localStorage.getItem('day');
        var curDay = new Date().getDate();
        if(tempService != null && !this.service.invalid){

          console.log("tubes service cached");
          this.service.data = JSON.parse(tempService);
          dfd.resolve(this.service.data);//
        } else{

          console.log("tubes service NOT cached");
          this.service.invalid = false;
          var srvc = this.service;
          var lan = JSON.parse(localStorage.getItem('userLanguage'));
          $http.get('http://api.tubesmash.com/api/v1/tubes?language='+lan.id+'&version=1.3.0')
            .success(function(data, status, headers, config) {
              localStorage.setItem('service', JSON.stringify(data));
              localStorage.setItem('day', curDay);
              //$timeout(function() {}, 3000);
              dfd.resolve(data);//
              srvc.data = data;
            })
            .error(function(data, status, headers, config) {
              displayWarning($filter('translate')('ops'), $filter('translate')('no_internet'));
              if(tempService){
                console.log("net error, fallback old data");
                srvc.data = JSON.parse(tempService);
                dfd.resolve(srvc.data);
              }else{

                var empty = {bestTubes:[], categories:[]};
                dfd.resolve(empty);
              }
            })
        }
      }else{
        this.service.invalid = false;
        if(this.service.data){
          console.log("ionic cached");
          dfd.resolve(this.service.data);
        }else{
          var tempService = localStorage.getItem('service');
          this.service.data = JSON.parse(tempService);
          dfd.resolve(this.service);//
        }
      }
      return dfd.promise;
    },

    getCountryIndex: function(countries){
      var dfd = $q.defer();
      $http.get('http://ip-api.com/json')
        .success(function(data, status, headers, config) {
          if(data.countryCode){
            var state = data.countryCode;
            
            for(var i=0, len=countries.length; i<len; i++) {
              if(countries[i].id == state.toLowerCase()){
                dfd.resolve(i);
                return;
              }
            }
          }
          dfd.resolve(countries.length-2);
          return;
        })
        .error(function(data, status, headers, config) {
          console.log(status);
          dfd.resolve(countries.length-2);
          return;
        })
      return dfd.promise;
    },

    startNotifications: function(countryId) {
      var set = function(data){
        var time = new Date(data.maxTime.year, data.maxTime.month, data.maxTime.day, data.maxTime.hours, data.maxTime.minutes, data.maxTime.seconds, data.maxTime.milliseconds);
        var timeToGo = new Date(data.time.year, data.time.month, data.time.day, data.time.hours, data.time.minutes, data.time.seconds, data.time.milliseconds);
        var now = new Date().getTime();
        var _when = new Date(now + data.after*1000);
        if(now < time.getTime()){
          if(timeToGo.getTime() > now){
            _when = timeToGo;
            console.log("timeToGo");
          }
          cordova.plugins.notification.local.schedule({
            id: data.id,
            title: data.title,
            text: data.text,
            smallIcon: data.smallIcon,
            at: _when,
            data: data.data
          });
        }
      }
      if(!document.notified){
        document.notified = true;
        var lan = JSON.parse(localStorage.getItem('userLanguage'));
        $http.get("http://tubesmash.com/api/v1/notifies?language="+lan.id)
          .success(function(data, status, headers, config) {
            $timeout(function(){
              
              cordova.plugins.notification.local.isPresent(data.id, function(l){
                
                if(l == false){
                  console.log("creo");
                  set(data);
                }
              })
            }, 5000, true);
          })
      }
    },

    getServiceResolved: function(){
      
      return this.getService();
    },

    getCategories: function() {
      return this.getServiceResolved().then(function(d){
        console.log("cate");
        return d.categories;
      })
    },

    getMostSearched: function() {
      return this.getServiceResolved().then(function(d){
        return d.mostSearched;
      })
    },

    getCategory: function(categoryId) {

      var dfd = $q.defer();
      var ts = this.getTubes;
      this.getServiceResolved().then(function(d){
        d.categories.forEach(function(category) {
          if (category.id == categoryId){
            //ts(category.tubes).then(function(tubes){
              //category.tube = tubes;
            //});
            dfd.resolve(category);
          } 
        })
      });
      return dfd.promise;
    },
   
    getTube: function(categoryId, tubeId) {
      var dfd = $q.defer();
      this.getServiceResolved().then(function(d){
        if(categoryId == "top"){
          d.bestTubes.forEach(function(tube) {

            if (tube.id == tubeId){

              dfd.resolve(tube);
            }
          })
        }
        d.categories.forEach(function(category) {
          if (category.id == categoryId){
            category.tubes.forEach(function(tube) {
              if (tube.id == tubeId){
                dfd.resolve(tube);
              }
            })
          }
        })
      })
      return dfd.promise;
    },

    getExtTube: function(tubeId) {
      return $http.get('http://tubesmash.com/api/v1/tubes/'+tubeId)
        .then(function(response) {
          return response.data;
        })
    },

    searchTubes: function(query) {
      var dfd = $q.defer();
      $http.get('http://tubesmash.com/api/v1/tubes/search?query='+query)
        .then(function(response) {
          if(!response.data.length)
            dfd.resolve([]);
          readyFn(response.data, dfd);
        })
      return dfd.promise;
    },

    getBestTubes: function() {
      var dfd = $q.defer();
      this.getServiceResolved().then(function(d){   

        var curTubes = d.bestTubes;
        if(!curTubes.length)
          dfd.resolve([]);
        if(ImgCache.ready){
          readyFn(curTubes, dfd);
        }else{
          if(ImgCache.cacheFailed){
            failFn(curTubes, dfd);
          }else{
            $rootScope.$on('ImgCacheStarted', function(){
              readyFn(curTubes, dfd);
            });
            $rootScope.$on('ImgCacheFailed',  function(){
              failFn(curTubes, dfd);
            });
          }
        }
      })
      return dfd.promise;
    },

    getTubes: function(tubes) {
      var dfd = $q.defer();
      var curTubes = tubes;
      if(!tubes.length)
        dfd.resolve([]);
      if(ImgCache.ready){

        readyFn(curTubes, dfd);
      }else{
        if(ImgCache.cacheFailed){
          failFn(curTubes, dfd);
        }else{

          $rootScope.$on('ImgCacheStarted', function(){
            readyFn(curTubes, dfd);
          });
          $rootScope.$on('ImgCacheFailed',  function(){
            failFn(curTubes, dfd);
          });
        }
      }
      return dfd.promise;
    },

    getMyTubes: function() {
      var dfd = $q.defer();
      starTubes = JSON.parse(localStorage.getItem('starTubes'));
      console.log(starTubes);
      if(!starTubes || starTubes === {}){
        
        starTubes = {};
      }
      var tubes = [];
      for (var t in starTubes) {

        if (starTubes.hasOwnProperty(t)) {
          tubes.push(starTubes[t]);
        }
      }
      if(!tubes.length)
        dfd.resolve([]);
      var curTubes = tubes;
      if(ImgCache.ready){

        readyFn(curTubes, dfd);
      }else{
        if(ImgCache.cacheFailed){
          failFn(curTubes, dfd);
        }else{

          $rootScope.$on('ImgCacheStarted', function(){
            readyFn(curTubes, dfd);
          });
          $rootScope.$on('ImgCacheFailed',  function(){
            failFn(curTubes, dfd);
          });
        }
      }
      return dfd.promise;
    },
  }
}); 


app.service('Cache', function( $timeout ){
  
  this.getFromVideoCache = function(input){
    $timeout(function(){
       ret = "after";
    },2000);
    return ret;
  }
})